@@ -69,7 +69,7 @@ def augments(cls_cpp: Type[Any]):
     wrapper classes and subclasses can become problematic if the call stack
     crosses the C++/Python boundary multiple times.
 
-    Any existing methods may be used, regardless of whether they defined
+    Any existing methods may be used, regardless of whether they are defined
     elsewhere in the support class or in the target class.
 
     For data fields to work, including @property accessors, the target class must be
@@ -79,13 +79,19 @@ def augments(cls_cpp: Type[Any]):
     This works on pure Python classes too.
 
     THIS DOES NOT work for class methods.
+
+    (Alternative ideas: https://github.com/pybind/pybind11/issues/1074)
     """
+    ATTR_WHITELIST = {'__repr__', '__enter__', '__exit__'}
 
     def class_augment(cls, cls_cpp=cls_cpp):
         for name, member in inspect.getmembers(cls):
-            if hasattr(cls_cpp, name):  # For PyPy: don't replace existing methods
+            # Don't replace existing methods except those in our whitelist
+            if hasattr(cls_cpp, name) and name not in ATTR_WHITELIST:
                 continue
             if inspect.isfunction(member):
+                if member.__qualname__.startswith('object.'):
+                    continue  # To avoid breaking PyPy
                 member.__qualname__ = member.__qualname__.replace(
                     cls.__name__, cls_cpp.__name__
                 )
@@ -93,11 +99,11 @@ def augments(cls_cpp: Type[Any]):
             elif inspect.isdatadescriptor(member):
                 setattr(cls_cpp, name, member)
 
-        def block_init(self):
+        def disable_init(self):
             # Prevent initialization of the support class
             raise NotImplementedError(self.__class__.__name__ + '.__init__')
 
-        cls.__init__ = block_init
+        cls.__init__ = disable_init
         return cls
 
     return class_augment
@@ -115,6 +121,7 @@ def _single_page_pdf(page) -> bytes:
 
 def _mudraw(buffer, fmt) -> bytes:
     """Use mupdf draw to rasterize the PDF in the memory buffer"""
+    # mudraw cannot read from stdin so NamedTemporaryFile is required
     with NamedTemporaryFile(suffix='.pdf') as tmp_in:
         tmp_in.write(buffer)
         tmp_in.seek(0)
@@ -126,14 +133,12 @@ def _mudraw(buffer, fmt) -> bytes:
             stderr=PIPE,
             check=True,
         )
-        if proc.stderr:
-            raise RuntimeError(proc.stderr.decode())
         return proc.stdout
 
 
 @augments(Object)
 class Extend_Object:
-    def _repr_mimebundle_(self, include, exclude, **kwargs):
+    def _repr_mimebundle_(self, include=None, exclude=None):
         """Present options to IPython for rich display of this object
 
         See https://ipython.readthedocs.io/en/stable/config/integrating.html#rich-display
@@ -144,7 +149,7 @@ class Extend_Object:
             and Name.Type in self
             and self.Type == Name.Page
         ):
-            return Page(self)._repr_mimebundle_(include, exclude, **kwargs)
+            return Page(self)._repr_mimebundle_(include, exclude)
         return None
 
     def _ipython_key_completions_(self):
@@ -152,7 +157,7 @@ class Extend_Object:
             return self.keys()
         return None
 
-    def emplace(self, other: Object):
+    def emplace(self, other: Object, retain=(Name.Parent,)):
         """Copy all items from other without making a new object.
 
         Particularly when working with pages, it may be desirable to remove all
@@ -160,6 +165,9 @@ class Extend_Object:
         of it, in a way that preserves all links and references to the original
         page. (Or similarly, for other Dictionary objects in a PDF.)
 
+        Any Dictionary keys in the iterable *retain* are preserved. By default,
+        /Parent is retained.
+
         When a page is assigned (``pdf.pages[0] = new_page``), only the
         application knows if references to the original the original page are
         still valid. For example, a PDF optimizer might restructure a page
@@ -179,8 +187,16 @@ class Extend_Object:
         """
         if not self.same_owner_as(other):
             raise TypeError("Objects must have the same owner for emplace()")
-        del_keys = set(self.keys()) - set(other.keys())
-        for k in other.keys():
+
+        # .keys() returns strings, so make all strings
+        retain = set(str(k) for k in retain)
+        self_keys = set(self.keys())
+        other_keys = set(other.keys())
+
+        assert all(isinstance(k, str) for k in (retain | self_keys | other_keys))
+
+        del_keys = self_keys - other_keys - retain
+        for k in (k for k in other_keys if k not in retain):
             self[k] = other[k]  # pylint: disable=unsupported-assignment-operation
         for k in del_keys:
             del self[k]  # pylint: disable=unsupported-delete-operation
@@ -274,7 +290,7 @@ class Extend_Pdf:
         warn("Pdf.root is deprecated; use Pdf.Root", category=DeprecationWarning)
         return self.Root
 
-    def _repr_mimebundle_(self, **_kwargs):
+    def _repr_mimebundle_(self, include=None, exclude=None):
         """
         Present options to IPython or Jupyter for rich display of this object
 
@@ -428,7 +444,7 @@ class Extend_Pdf:
         self, *, page_size: Tuple[Numeric, Numeric] = (612.0, 792.0)
     ) -> Object:
         """
-        Add a blank page to this PD. If pages already exist, the page will be added to
+        Add a blank page to this PDF. If pages already exist, the page will be added to
         the end. Pages may be reordered using ``Pdf.pages``.
 
         The caller may add content to the page by modifying its objects after creating
@@ -497,10 +513,7 @@ class Extend_Pdf:
             b"%%EOF\n"
         )
 
-        if self.filename:
-            description = "closed file: " + self.filename
-        else:
-            description = "closed object"
+        description = "closed file: " + self.filename
         self._process(description, EMPTY_PDF)
         if getattr(self, '_tmp_stream', None):
             self._tmp_stream.close()
@@ -554,19 +567,12 @@ class Extend_Pdf:
 
         problems: List[str] = []
 
-        try:
-            self._decode_all_streams_and_discard()
-        except PdfError as e:
-            problems.append(str(e))
+        self._decode_all_streams_and_discard()
 
         discarding_parser = DiscardingParser()
-
         for basic_page in self.pages:
             page = Page(basic_page)
-            try:
-                page.parse_contents(discarding_parser)
-            except PdfError as e:
-                problems.append(str(e))
+            page.parse_contents(discarding_parser)
 
         for warning in self.get_warnings():
             problems.append("WARNING: " + warning)
@@ -771,13 +777,13 @@ class Extend_Pdf:
 
             :meth:`pikepdf.Pdf.remove_unreferenced_resources` before saving
             may eliminate unnecessary resources from the output file if there
-            any objects (such as images) that are referenced in a page's
+            are any objects (such as images) that are referenced in a page's
             Resources dictionary but never called in the page's content stream.
 
         .. note::
 
-            pikepdf can read PDFs will incremental updates, but always
-            any coalesces incremental updates into a single non-incremental
+            pikepdf can read PDFs with incremental updates, but always
+            coalesces any incremental updates into a single non-incremental
             PDF file when saving.
         """
         if not filename_or_stream and self._original_filename:
@@ -822,11 +828,11 @@ class Extend_Pdf:
         lazily loaded. Despite this restriction, pikepdf does not try to use any OS
         services to obtain an exclusive lock on the file. Some applications may
         want to attempt this or copy the file to a temporary location before
-        editing. This behaviour change if *allow_overwriting_input* is set: the whole
+        editing. This behaviour changes if *allow_overwriting_input* is set: the whole
         file is then read and copied to memory, so that pikepdf can overwrite it
         when calling ``.save()``.
 
-        When this is function is called with a stream-like object, you must ensure
+        When this function is called with a stream-like object, you must ensure
         that the data it returns cannot be modified, or undefined behavior will
         occur.
 
@@ -893,6 +899,16 @@ class Extend_Pdf:
                 usable.
             FileNotFoundError: If the file was not found.
         """
+        if isinstance(filename_or_stream, bytes) and filename_or_stream.startswith(
+            b'%PDF-'
+        ):
+            warn(
+                "It looks like you called with Pdf.open(data) with a bytes-like object "
+                "containing a PDF. This will probably fail because this function "
+                "expects a filename or opened file-like object. Instead, please use "
+                "Pdf.open(BytesIO(data))."
+            )
+
         tmp_stream, original_filename = None, False
         if allow_overwriting_input:
             try:
@@ -923,14 +939,6 @@ class Extend_Pdf:
 
 @augments(_ObjectMapping)
 class Extend_ObjectMapping:
-    def __contains__(self, key) -> bool:
-        try:
-            self[key]
-        except KeyError:
-            return False
-        else:
-            return True
-
     def get(self, key, default=None) -> Object:
         try:
             return self[key]
@@ -1052,9 +1060,9 @@ class Extend_Page:
         return name
 
     def __repr__(self):
-        return repr(self.obj).replace('Dictionary', 'Page')
+        return repr(self.obj).replace('Dictionary', 'Page', 1)
 
-    def _repr_mimebundle_(self, include, exclude, **_kwargs):
+    def _repr_mimebundle_(self, include=None, exclude=None):
         data = {}
         bundle = {'application/pdf', 'image/png'}
         if include:
